/*************************************************
 * Copyright (C) 2009 Aviad Zuck & Sivan Toledo
 * This file is part of NANDFS.
 *
 * To license NANDFS under a different license, please
 * contact the authors.
 *
 * NANDFS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * NANDFS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with NANDFS.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef OPENTESTS_H_
#define OPENTESTS_H_

#include <system.h>

void runAllOpenTests(void);

/**
 * @brief 
 * init open test
 * 
 */
void init_openTest(void);

/**
 * @brief 
 * tear down open test
 * 
 */
void tearDown_openTest(void);

/**
 * @brief
 * open a file that already exists for reading.
 * should succeed
 * @return 1 if successful, 0 otherwise
 */
error_t openTest1(void);

/**
 * @brief
 * open a file that doesn't exists for reading.
 * should fail
 * @return 1 if successful, 0 otherwise
 */
error_t openTest2(void);

/**
 * @brief
 * open a file that exists for writing.
 * should succeed
 * @return 1 if successful, 0 otherwise
 */
error_t openTest3(void);

/**
 * @brief
 * open a file that doesn't exists for writing
 * should fail
 * @return 1 if successful, 0 otherwise
 */
error_t openTest4(void);

/**
 * @brief
 * open a file that exists twice for reading
 * should succeed
 * @return 1 if successful, 0 otherwise
 */
error_t openTest5(void);

/**
 * @brief
 * open a file for writing, then try to open it to reading
 * should fail (write exclusive)
 * @return 1 if successful, 0 otherwise
 */
error_t openTest6(void);

/**
 * @brief
 * open a file that exists for reading, the try to  open for writing. 
 * should fail (write exclusive)
 * @return 1 if successful, 0 otherwise
 */
error_t openTest7(void);

/**
 * @brief
 * open FS_MAX_VNODES different files for reading , than another file and making sure
 * the last open fails with the matching error 
 * @return 1 if successful, 0 otherwise
 */
error_t openTest8(void);

/**
 * @brief
 * open with different users FS_MAX_OPEN_FILES open file entries (for less than FS_MAX_VNODES files),
 * than another file and making sure  the last open fails with 
 * the matching error 
 * @return 1 if successful, 0 otherwise
 */
error_t openTest9(void);

/**
 * @brief
 * try opening a pathname that leads to a directory with write permission.
 * should fail
 * @return 1 if successful, 0 otherwise
 */
error_t openTest23(void);

/**
 * @brief
 * try opening a pathname which has a non existing directory in the middle
 * should fail
 * @return 1 if successful, 0 otherwise
 */
error_t openTest24(void);

/**
 * @brief
 * try opening a pathname which has a non existing directory in the end
 * should fail
 * @return 1 if successful, 0 otherwise
 */
error_t openTest25(void);

/**
 * @brief
 * try opening a legal pathname with "." and ".." 
 * @return 1 if successful, 0 otherwise
 */
error_t openTest26(void);

#endif /*OPENTESTS_H_*/
